package org.rcisoft.business.bchapter.service.impl;

import org.apache.commons.io.FileUtils;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.dto.*;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.AutomaticEnum;
import org.rcisoft.business.bchapter.enums.ChapterLevelEnum;
import org.rcisoft.business.bchapter.enums.HasMarkEnum;
import org.rcisoft.business.bchapter.enums.StateEnum;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bchapter.vo.StudentChapterScoreVO;
import org.rcisoft.business.bcode.model.IntellEvaluation;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.brslstudent.dao.BRSlStudentRepository;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bsl.enums.AuditStatusEnum;
import org.rcisoft.business.bstudent.dao.BStudentRepository;
import org.rcisoft.business.bfile.dao.BFileRepository;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BChapterServiceImpl implements BChapterService {
    @Autowired
    private BChapterRepository bChapterRepository;
    @Autowired
    private BSlRepository bSlRepository;
    @Autowired
    private BStudentRepository bStudentRepository;
    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    private BFileRepository bFileRepository;
    @Autowired
    private Global global;

    @Override
    public BChapter selectOne(String businessId){
        BChapter result = bChapterRepository.selectByPrimaryKey(businessId);
        if(result==null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);
        return result;
    }

    @Override
    public List<BChapter> queryBChaptersByPagination(PageUtil pageUtil, BChapter model) {
        Map param = new HashMap<String,Object>();
        return bChapterRepository.queryBChapters(param);
    }

    @Override
    public List<StudentChapterScoreVO> queryScoreListByChapterIdByPagination(PageUtil pageUtil, QueryScoreListDTO dto){
        return bChapterRepository.queryScoreListByChapterId(dto);
    }

    @Override
    public List<BChapter> queryBChapters(BChapter model) {
        Map param = new HashMap<String,Object>();
        if(model.getSlId()!=null&&!"".equals(model.getSlId())){
            param.put("slId",model.getSlId());
        }
        if(model.getLessonId()!=null&&!"".equals(model.getLessonId())){
            param.put("lessonId",model.getLessonId());
        }
        param.put("lessonOrSl",model.getLessonOrSl());
        return bChapterRepository.queryBChapters(param);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BChapter model, String token){
        int line = 0;
        if(model.getBusinessId()==null) {
            UserUtil.setCurrentPersistOperation(model);
            if(model.getClassHour()==null||model.getClassHour().equals("")){
                model.setClassHour("");
            }
            if(model.getIsTest()==null||model.getIsTest().equals("")){
                model.setIsTest("");
            }
            int sort = bChapterRepository.selectMaxSort(model.getPid(),model.getSlId(),model.getLessonId());
            model.setSort(sort);
            line = bChapterRepository.insertSelective(model);
        }else {
            UserUtil.setCurrentMergeOperation(model);
            line = bChapterRepository.updateByPrimaryKeySelective(model);
        }
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel update(BChapter model) {
        int line = bChapterRepository.updateByPrimaryKeySelective(model);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBChapter(String id,String lessonOrSl) {
        BChapter bChapter = bChapterRepository.selectByPrimaryKey(id);
        if(bChapter==null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);
        List<BChapter> childList = bChapterRepository.queryBChaptersByPid(id);
        boolean hasChild =childList!=null&&childList.size()>0;
        boolean isChapter = bChapter.getChapterLevel()== ChapterLevelEnum.CHAPTER.getCode();
        if(isChapter&&hasChild) {
            throw new ServiceException(ResultServiceEnums.CHAPTER_HAS_CHILD);
        }
        String state;
        if(lessonOrSl.equals("0")){
            state = bSlRepository.queryStateById(bChapter.getSlId());
            if(state.equals(AuditStatusEnum.PUBLISHED.getCode())){
                throw new ServiceException(ResultServiceEnums.CHAPTER_IS_USED);
            }
        }
        bChapterRepository.deleteStudentChapter(id);
        int line = bChapterRepository.deleteByPrimaryKey(id);
        return new PersistModel(line);
    }

    @Override
    @Transactional
    public void markScore(List<ScoreInfoDTO> scoreInfoDTOS,String token) {
        for(ScoreInfoDTO scoreInfoDTO:scoreInfoDTOS){
            String id = bChapterRepository.selectIdByStudentIdAndChapterId(scoreInfoDTO.getChapterId(),scoreInfoDTO.getStudentId());
            if(scoreInfoDTO.getScore()==null||scoreInfoDTO.getScore()<0||scoreInfoDTO.getScore()>100)  //分数不为空且在0到100之间
                throw new ServiceException(ResultServiceEnums.SCORE_ERROR);
            if(id==null){   //查看该学生该章节是否已经完成学习
                  throw new ServiceException(ResultServiceEnums.UN_COMPLETE);
            }else{   //已经学习则修改
                scoreInfoDTO.setBusinessId(id);
                scoreInfoDTO.setAutomatic("0");
                UserUtil.setCurrentMergeOperation(scoreInfoDTO);
                bChapterRepository.updateScoreInfo(scoreInfoDTO);
            }
        }
    }

    @Override
    public List<StuScoreDTO> queryScoreListBySlIdAndStuId(String slid, String stuid) {
        return bChapterRepository.selectScoreListBySlIdAndStuId(slid, stuid);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public void deleteScoerInfo(String chapterId, String studentCode) {
        String studentId = bStudentRepository.queryStudentIdByCode(studentCode);
        int result = bChapterRepository.updateScoerInfo(chapterId,studentId);
        if(result<1)
            throw new ServiceException(ResultServiceEnums.SCORE_INFO_NOT_EXISTS);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public void updateIsComplete(ScoreInfoDTO scoreInfoDTO,String token) throws Exception{
        String stuid = bStudentRepository.queryStudentByUserId(scoreInfoDTO.getStudentId());

        scoreInfoDTO.setCommonBusinessId();
        UserUtil.setCurrentMergeOperation(scoreInfoDTO);

        BChapter bChapter = bChapterRepository.selectOne(new BChapter(scoreInfoDTO.getChapterId()));
        if(bChapter.getAutomatic().equals("1")){//智评
            String codeType = global.getCOURSE_CODE_LOCATION();
            if(bChapter.getExperimentType().equals("2")||bChapter.getExperimentType().equals("4")){
                codeType = global.getCOURSE_PROJECT_LOCATION();
            }
            String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                    + global.getCOURSE_LOCATION() + File.separator
                    + global.getSL_LOCATION() + File.separator
                    + scoreInfoDTO.getSlId() + File.separator
                    + scoreInfoDTO.getChapterId() + File.separator
                    + codeType + File.separator
                    + scoreInfoDTO.getStudentId() + File.separator;
            //获取智评分数
            int score = IntellEvaluation.IntellEvaluation(filePath,scoreInfoDTO.getStudentCode(),bChapter.getExperimentType());
            scoreInfoDTO.setScore(score);

        }else{
            scoreInfoDTO.setScore(-1);
        }
        scoreInfoDTO.setStudentId(stuid);
        int result;
        //查询是否已有关联关系
        ScoreInfoDTO studentChapter = bChapterRepository.queryStudentChapter(scoreInfoDTO.getChapterId(),stuid);
        if(studentChapter!=null){//update
            result = bChapterRepository.updateToFinish(scoreInfoDTO);
        }else{//insert
            result = bChapterRepository.updateIsComplete(scoreInfoDTO);
        }

        if(bChapter.getExperimentType().equals("2")||bChapter.getExperimentType().equals("4")){

        }
        if(result<1)
            throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
    }

    @Override
    public ScoreInfoDTO getChapInfoById(ScoreInfoDTO scoreInfoDTO) throws Exception {

        return bChapterRepository.queryChapInfoById(scoreInfoDTO.getChapterId(),scoreInfoDTO.getStudentId());
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int addBslFormLesson(String bLessonId, String lessonCode, String slId) {
        int result = 0;
        if(bLessonId==null){
            BLesson bLseeon = bLessonRepository.queryBLessonByCode(lessonCode);
            bLessonId = bLseeon.getBusinessId();
        }
        //获取课程的章节列表
        List<BChapter> lessonList = bChapterRepository.queryChapterByLessonId(bLessonId);
        List<BChapter> chapterList = new ArrayList<>();
        //map存放章的ID（key:老的章id value:新的章ID）
        Map map = new HashMap();
        for (int i=0;i<lessonList.size();i++){
            BChapter sl_chapter = new BChapter();
            sl_chapter.setSlId(slId);
            sl_chapter.setChapterName(lessonList.get(i).getChapterName());
            sl_chapter.setChapterLevel(lessonList.get(i).getChapterLevel());
            sl_chapter.setChapterUrl(lessonList.get(i).getChapterUrl());
            sl_chapter.setMdFile(lessonList.get(i).getMdFile());
            sl_chapter.setIsTest(lessonList.get(i).getIsTest());
            sl_chapter.setClassHour(lessonList.get(i).getClassHour());
            sl_chapter.setState(StateEnum.CLOSE.getCode());
            sl_chapter.setSort(lessonList.get(i).getSort());
            sl_chapter.setExperimentType(lessonList.get(i).getExperimentType());
            sl_chapter.setAutomatic("1");
            sl_chapter.setLessonOrSl("0");
            UserUtil.setCurrentPersistOperation(sl_chapter);
            if(lessonList.get(i).getPid().equals("-1")){
                sl_chapter.setPid(lessonList.get(i).getPid());
                map.put(lessonList.get(i).getBusinessId(),sl_chapter.getBusinessId());
            }else{
                sl_chapter.setPid("0");
                sl_chapter.setLPid(lessonList.get(i).getPid());
            }
            //set老的章节ID后面比对用
            lessonList.get(i).setSPid(lessonList.get(i).getBusinessId());
            sl_chapter.setSPid(lessonList.get(i).getBusinessId());

            chapterList.add(sl_chapter);
        }
        for (int i=0;i<chapterList.size();i++){
            if(!chapterList.get(i).getPid().equals("-1")){
                //取出map中存放的新的章ID作为父节点ID
                chapterList.get(i).setPid(String.valueOf(map.get(chapterList.get(i).getLPid())));
            }
        }

        if(chapterList.size()>0){
            result = bChapterRepository.insertChapterList(chapterList);
        }

        String oldPath = global.getBASE_UPLOAD_SERVER_LOCATION()+File.separator
                +global.getCOURSE_LOCATION()+File.separator
                +global.getLESSON_LOCATION()+File.separator
                +bLessonId;
        String newPath = global.getBASE_UPLOAD_SERVER_LOCATION()+File.separator
                +global.getCOURSE_LOCATION()+File.separator
                +global.getSL_LOCATION()+File.separator
                +slId;

        //复制章节的所有文件
        try {
            //copyFileToSl(oldPath,newPath,bLessonId,slId);
            File oldFile = new File(oldPath);
            File newFile = new File(newPath);
            FileUtils.copyDirectory(oldFile,newFile,true);
        } catch (IOException e) {
            e.printStackTrace();
        }

        //数据库操作
        for (BChapter leBChapter : lessonList){
            for(BChapter slChapter : chapterList){
                if(leBChapter.getSPid().equals(slChapter.getSPid())&&!leBChapter.getPid().equals("-1")){
                    BFile bFile = bFileRepository.queryBVideoByChapterId(leBChapter.getBusinessId());
                    BFile newVideo = new BFile();
                    newVideo.setChapterId(slChapter.getBusinessId());
                    UserUtil.setCurrentPersistOperation(newVideo);
                    //将节的ID替换成新的
                    if(bFile !=null){
                        newVideo.setVideoName(bFile.getVideoName());
                        if(bFile.getVideoUrl()!=null&&!bFile.getVideoUrl().equals("")){
                            String newUrl = bFile.getVideoUrl().replace(bLessonId,slId);
                            String path = newUrl.replace(leBChapter.getBusinessId(),slChapter.getBusinessId());
                            newVideo.setVideoUrl(path.replace(global.getLESSON_LOCATION(), global.getSL_LOCATION()));
                        }
                        if(bFile.getFileUrl()!=null&&!bFile.getFileUrl().equals("")){
                            String newUrl = bFile.getFileUrl().replace(bLessonId,slId);
                            String path = newUrl.replace(leBChapter.getBusinessId(),slChapter.getBusinessId());
                            newVideo.setFileUrl(path.replace(global.getLESSON_LOCATION(), global.getSL_LOCATION()));
                        }
                        if(bFile.getPptUrl()!=null&&!bFile.getPptUrl().equals("")){
                            String newUrl = bFile.getPptUrl().replace(bLessonId,slId);
                            String path = newUrl.replace(leBChapter.getBusinessId(),slChapter.getBusinessId());
                            newVideo.setPptUrl(path.replace(global.getLESSON_LOCATION(), global.getSL_LOCATION()));
                        }
                        if(bFile.getPdfUrl()!=null&&!bFile.getPdfUrl().equals("")){
                            String newUrl = bFile.getPdfUrl().replace(bLessonId,slId);
                            String path = newUrl.replace(leBChapter.getBusinessId(),slChapter.getBusinessId());
                            newVideo.setPdfUrl(path.replace(global.getLESSON_LOCATION(), global.getSL_LOCATION()));
                        }
                        //重命名复制之后的文件
                        String filePath = newPath + File.separator + leBChapter.getBusinessId();
                        String newFilePath = newPath + File.separator + slChapter.getBusinessId();
                        File file = new File(filePath);
                        file.renameTo(new File(newFilePath));
                        //插入对象
                        newVideo.setSlId(slId);
                        bFileRepository.insertSelective(newVideo);
                    }
                }
            }
        }
        return result;
    }

    /**
     * 递归方法复制文件夹及子目录
     * @param oldPath
     * @param newPath
     */
    private void copyFileToSl(String oldPath, String newPath, String bLessonId, String slId) throws IOException {

        File oldFile = new File(oldPath);
        File[] fs=oldFile.listFiles();
        File newFile=new File(newPath);
        if(!newFile.exists()){
            newFile.mkdirs();
        }
        for (File f : fs) {
            if(f.isFile()){
                fileCopy(f.getPath(),newPath+File.separator+f.getName()); //调用文件拷贝的方法
            }else if(f.isDirectory()){
                copyFileToSl(f.getPath(),newPath+File.separator+f.getName(), bLessonId, slId);
            }
        }
    }

    private void fileCopy(String oldPath, String newPath) {
        BufferedReader br=null;
        PrintStream ps=null;
        try {
            br=new BufferedReader(new InputStreamReader(new FileInputStream(oldPath)));
            ps=new PrintStream(new FileOutputStream(newPath));
            String s=null;
            while((s=br.readLine())!=null){
                ps.println(s);
                ps.flush();
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally{
            try {
                if(br!=null)  br.close();
                if(ps!=null)  ps.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int updateChapterState(BChapter bChapter) {
        if(bChapter.getState().equals(StateEnum.OPEN.getCode())){
            bChapter.setState(StateEnum.CLOSE.getCode());
        }else
            bChapter.setState(StateEnum.OPEN.getCode());
        int result = bChapterRepository.updateStateById(bChapter);
        if(!bChapter.getPid().equals("-1")){
            List<String> state = bChapterRepository.queryChapterStateByPid(bChapter.getPid());
            if(state.size()==1&&state.get(0).equals(StateEnum.CLOSE.getCode())){
                bChapterRepository.updateStateByPid(bChapter.getPid(),StateEnum.CLOSE.getCode());
            }else if(state.size()>1){
                bChapterRepository.updateStateByPid(bChapter.getPid(),StateEnum.OPEN.getCode());
            }
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int updateAllChapterState(String slId) {
        List<String> list = bChapterRepository.selectStateBySlId(slId);
        String state = StateEnum.OPEN.getCode();
        if (list.size() == 1 && list.get(0).equals(state)) {
            state = StateEnum.CLOSE.getCode();
        }
        int result = bChapterRepository.updateAllStateBySlId(slId, state);
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int updateTopChapter(BChapter bChapter) {
        int result = 0;
        BChapter lessChapter = bChapterRepository.getlessChapter(bChapter);
        int sort = bChapter.getSort();
        int lessSort = lessChapter.getSort();
        int data = bChapterRepository.updateSortById(bChapter.getBusinessId(),lessSort);
        int data2 = bChapterRepository.updateSortById(lessChapter.getBusinessId(),sort);
        if(data==data2){
            result = 1;
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int updateDownChapter(BChapter bChapter) {
        int result = 0;
        BChapter bigChapter = bChapterRepository.getBigChapter(bChapter);
        int sort = bChapter.getSort();
        int bigSort = bigChapter.getSort();
        int data = bChapterRepository.updateSortById(bChapter.getBusinessId(),bigSort);
        int data2 = bChapterRepository.updateSortById(bigChapter.getBusinessId(),sort);
        if(data==data2){
            result = 1;
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int updateAutomatic(String businessId) {
        String nowAutomatic = bChapterRepository.selectAutomatic(businessId);
        String automatic = AutomaticEnum.AUTOMATIC_YES.getCode();
        if(nowAutomatic.equals(AutomaticEnum.AUTOMATIC_YES.getCode())){
            automatic = AutomaticEnum.AUTOMATIC_NO.getCode();
        }
        int result = bChapterRepository.updateAutomatic(automatic,businessId);
        return result;
    }

    @Override
    public String queryMdFileByChapterId(String businessId) {
        return bChapterRepository.queryMdFileByChapterId(businessId);
    }

    @Override
    public List<BChapter> queryBChaptersByStudent(BChapter param) {
        return bChapterRepository.queryBChaptersByStudent(param);
    }

    @Override
    public int Mark(String chapterId, String slId, String studentCode, String score) {
        int line;
        ScoreInfoDTO scoreInfoDTO = bChapterRepository.queryScoreByChapterId(chapterId,studentCode);
        if(scoreInfoDTO!=null){//重新打分
            scoreInfoDTO.setAutomatic("0");
            line=bChapterRepository.remark(chapterId,scoreInfoDTO.getStudentId(),score);
        }else {//打分
            BStudent student = bStudentRepository.selectOne(new BStudent(studentCode));
            line=bChapterRepository.insertStudentScore(IdGen.uuid(),chapterId,slId,student.getBusinessId(),score);
        }

        return line;
    }

    @Override
    public List<ScoreReportDto> scoreReport(String slId) {
        return bChapterRepository.queryBChaptersBySlId(slId);
    }

    @Override
    public List<ChapterSelectDto> chapterListBySlId(String slId) {
        return bChapterRepository.chapterListBySlId(slId);
    }

    @Override
    public String queryStuIdByCode(String code) {
        return bChapterRepository.queryStuIdByCode(code);
    }

    @Override
    public int updateToFinish(String type, String studentCode, String chapterId ,String slId) {
        ScoreInfoDTO scoreInfoDTO = new ScoreInfoDTO();
        scoreInfoDTO.setChapterId(chapterId);
        String stuId = bChapterRepository.queryStuIdByCode(studentCode);
        scoreInfoDTO.setStudentId(stuId);
        if(type.equals("1")){
            scoreInfoDTO.setPptFinish("1");
        }else if(type.equals("2")){
            scoreInfoDTO.setPdfFinish("1");
        }else if(type.equals("3")){
            scoreInfoDTO.setVideoFinish("1");
        }else if(type.equals("4")){
            scoreInfoDTO.setPaperFinish("1");
        }
        //查询是否已有关联关系
        ScoreInfoDTO studentChapter = bChapterRepository.queryStudentChapter(chapterId,stuId);
        if(studentChapter!=null){//update
            scoreInfoDTO.setBusinessId(studentChapter.getBusinessId());
            return bChapterRepository.updateToFinish(scoreInfoDTO);
        }else{//insert
            scoreInfoDTO.setSlId(slId);
            scoreInfoDTO.setScore(-1);
            UserUtil.setCurrentPersistOperation(scoreInfoDTO);
            return bChapterRepository.insertStudentChapter(scoreInfoDTO);
        }

    }

    @Override
    public String queryIsCompleteByUserIdAndChapId(String userId, String chapId) {
        return bChapterRepository.queryIsCompleteByUserIdAndChapId(userId, chapId);
    }


}
