package org.rcisoft.business.bclass.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bclass.entity.BClass;
import org.rcisoft.business.bclass.service.BClassService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

/**
* Created by CodeGenerator on 2017/07/12.
*/
@RestController
@RequestMapping("/BClass")
public class BClassController extends PaginationController<BClass> {
    @Autowired
    private BClassService bClassService;

    @Autowired
    private Global global;


    @ApiOperation(value="添加/编辑班级", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "班级编号(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "className", value = "班级名称(1-150)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentNum", value = "学生数量(1-3)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(@Valid BClass bClass, BindingResult bindingResult) {
        if(bClass.getCode().length()>
                Integer.parseInt(global.getMAX_CODE_LENGTH()))
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
        PersistModel data = bClassService.persist(bClass,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bClass);
    }

    @ApiOperation(value="删除班级", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bClassService.removeBClass(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="excel导入", notes="上传excel到服务器")
    @ApiImplicitParam(name = "importFile", value = "excel文件", required = true, dataType = "MultipartFile")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile, String token) throws Exception {
        PersistModel data = bClassService.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile),token);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                null);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bClassService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询班级", notes="根据URL中的参数分页查询班级")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "班级编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "className", value = "班级名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "studentNum", value = "学生数量", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBClasssByPagination")
    public GridModel queryBClasssByPagination(BClass param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bClassService.queryBClasssByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询班级", notes="根据URL中的参数查询全部班级")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "班级编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "className", value = "班级名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "studentNum", value = "学生数量", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBClasss")
    public Result queryBClasss(BClass param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bClassService.queryBClasss(param));
    }

    @ApiOperation(value="查询班级列表", notes="查询班级列表赋值在下拉框中")
    @GetMapping(value = "/queryBClassList")
    public Result queryBClassList() {
        List<SelectModel> bClass = bClassService.queryBClassList();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bClass);
    }
}
