package org.rcisoft.business.bclass.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.bclass.dao.BClassRepository;
import org.rcisoft.business.bclass.entity.BClass;
import org.rcisoft.business.bclass.service.BClassService;
import org.rcisoft.business.brclassstudent.dao.BRClassStudentRepository;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BClassServiceImpl implements BClassService {
    @Autowired
    private BClassRepository bClassRepository;
    @Autowired
    private BRClassStudentRepository brClassStudentRepository;

    @Override
    public BClass selectOne(String businessId){
        return bClassRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BClass> queryBClasssByPagination(PageUtil pageUtil, BClass model) {
        return bClassRepository.queryBClasss(model);
    }

    @Override
    public List<BClass> queryBClasss(BClass model) {
        return bClassRepository.queryBClasss(model);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BClass model, String token){
        int line = 0;
        if(model.getBusinessId()==null) {
            if (bClassRepository.selectOne(new BClass(model.getCode())) != null)
                throw new ServiceException(ResultServiceEnums.CLASS_EXISTS);
            UserUtil.setCurrentPersistOperation(model);
            line = bClassRepository.insertSelective(model);
        }else
            UserUtil.setCurrentMergeOperation(model);
            line = bClassRepository.updateByPrimaryKeySelective(model);
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBClass(String id) {
        if(brClassStudentRepository.selectStuentNumByClassId(id)>0) {
            throw new ServiceException(ResultServiceEnums.CLASS_HAS_STUDENTS);
        }
        int line = bClassRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel importExcel(HSSFWorkbook hwb, String token) throws IOException {
        ArrayList<BClass> bClasses = new ArrayList<BClass>();
        ArrayList<String> repeatCode = new ArrayList<String>();

        String[] headers = {"班级编号","班级名称"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers);  //获取excel数据

        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        for(String[] value:values){   //将数据封装到entity
            BClass bClass = null;
            BClass b = new BClass(value[0]);
            if(value[0]==null||value[0].equals("")||value[1]==null||value[1].equals("")){
                continue;
            }
            if((bClass=bClassRepository.selectOne(b))!=null) {
                repeatCode.add(value[0]);
                continue;
            }
            bClass= new BClass(value[0],value[1]);
            bClass.setCommonBusinessId();
            UserUtil.setCurrentPersistOperation(bClass);

            if(!bClasses.contains(bClass)){
                bClasses.add(bClass);
            }
        }

        if(bClasses.size()<1){
            if(repeatCode.size()<1)
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
            else
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),"Excel内所有数据之前都已经导入:"+ JSON.toJSONString(repeatCode));
        }

        String result = ExcelUtil.checkRepeat(repeatCode);

        int line = bClassRepository.insertList(bClasses);
        return new PersistModel(line,result);
    }

    @Override
    public List<SelectModel> queryBClassList() {
        List<BClass> bClass = bClassRepository.queryBClassList();
        ArrayList<SelectModel> comBoxModel = new ArrayList<SelectModel>();
        for(BClass c : bClass){
            SelectModel selectModel = new SelectModel();
            selectModel.setId(c.getCode());
            selectModel.setText(c.getClassName());
            comBoxModel.add(selectModel);
        }
        return comBoxModel;
    }

    private Map<String,Object> queryParamHandler(BClass model){
        Map param = new HashMap<String, Object>();
        if(!model.getClassName().equals("")&&model.getClassName()!="")
            param.put("name","%"+model.getClassName()+"%");
        else
            param.put("name","%%");
        if(model.getStudentNum()!=null&&!model.getStudentNum().equals(""))
            param.put("num", Integer.parseInt(model.getStudentNum()));
        else
            param.put("num",null);
        return param;
    }
}
