package org.rcisoft.business.bcode.controller;

import io.swagger.annotations.*;
import org.apache.commons.io.FileUtils;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bchapter.service.impl.BChapterServiceImpl;
import org.rcisoft.business.bcode.model.BCodeFile;
import org.rcisoft.business.bcode.service.BCodeService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.CompileUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;

/**
 * Created by lcy on 18/1/3.
 *
 * 编程controller
 */
@RestController
@RequestMapping("/BCode")
public class BCodeController extends PaginationController {


    @Autowired
    private BCodeService bCodeServiceImpl;

    @Autowired
    private BChapterService bChapterService;

    @Autowired
    private Global global;

    /**
     * 学生看自己的实验
     * @param slId
     * @param chapId
     * @param examType
     * @return
     */
    @ApiOperation(value="查看实验代码", notes="学生看自己的实验")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "examType", value = "类型 0001:JAVA单文件，0002:JAVA工程，1001:HTML单文件，1002:HTML工程", required = true, dataType = "varchar")})
    @GetMapping(value = "/queryOwnExamFiles")
    public Result queryOwnExamFiles(@RequestParam("slId") String slId,
                                    @RequestParam("chapId") String chapId,
                                    @RequestParam("examType") String examType){
        List<BCodeFile> list = bCodeServiceImpl.readExamFiles(slId, chapId, examType,
                UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID),
                UserUtil.getUserInfoProp(getToken(), UserUtil.USER_USERNAME));
        if(examType.equals("0001")||examType.equals("1001"))
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    list.get(0));
        else
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    list);

    }



    @ApiOperation(value="实验是否启动", notes="实验是否启动")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节ID", required = true, dataType = "varchar")})
    @GetMapping(value = "/isStartedExam")
    public Result isStartedExam(@RequestParam("slId") String slId,
                                    @RequestParam("chapId") String chapId){

        Integer result = bCodeServiceImpl.isOpenLxcFromUser(UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID), slId, chapId);
        return Result.builder(new PersistModel(result),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                result);

    }

    @ApiOperation(value="关闭实验", notes="关闭实验")
    @GetMapping(value = "/stopExam")
    public Result stopExam(){
        CommandResult result = bCodeServiceImpl.stopExam(UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID));
        return Result.builder(new PersistModel(result.isSuccess()? 1 : 0),
                result.getMessage(),
                result.getMessage(),
                result);
    }

    @ApiOperation(value="延长实验", notes="延长实验")
    @GetMapping(value = "/delayExam")
    public Result delayExam(){
        CommandResult result = bCodeServiceImpl.delayExam(UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID));
        return Result.builder(new PersistModel(result.isSuccess()? 1 : 0),
                result.getMessage(),
                result.getMessage(),
                result);
    }





    @ApiOperation(value="保存实验代码", notes="学生保存自己的实验")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "content", value = "代码", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "examType", value = "类型 0001:JAVA单文件，0002:JAVA工程，1001:HTML单文件，1002:HTML工程", required = true, dataType = "varchar")})
    @PostMapping(value = "/saveOwnExamFiles")
    public Result saveOwnExamFiles(@RequestParam("slId") String slId,
                                   @RequestParam("chapId") String chapId,
                                   @RequestParam("examType") String examType,
                                   @RequestParam("content") String content){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCodeServiceImpl.saveOwnExamFiles(slId, chapId, examType, content,
                        UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID),
                        UserUtil.getUserInfoProp(getToken(), UserUtil.USER_USERNAME)
                ));
    }


    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "开课编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentId", value = "学生Id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "是否全部导出 1是全部导出  0否", required = true, dataType = "varchar")})
    @ApiOperation(value="导出学生代码文件", notes="根据学生Id导出学生代码文件")
    @GetMapping(value = "/exportFilesWithoutAuth")
    public void exportFilesWithoutAuth(String slId, String studentId, String type, String slCode, HttpServletResponse response)throws Exception{
        String fileName = bCodeServiceImpl.exportFiles(slId, studentId, type, slCode);
        this.downloadFiles(fileName,response);
    }


    public void downloadFiles(String fileName, HttpServletResponse response) throws Exception{
        ServletOutputStream out = null;
        InputStream in = null;
        String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getTEMP_LOCATION() + File.separator
                + global.getSTUDENT_CODE_LOCATION() + File.separator
                + fileName;
        try{
            File file = new File(path);
            if(!file.exists()){
                throw new ServiceException(ResultServiceEnums.DOWNLOAD_ERROR);
            }
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/force-download");
            response.addHeader("Content-Disposition","attachment;filename="+fileName);
            out = response.getOutputStream();
            in = new FileInputStream(file);
            byte[] buffer = new byte[1024];
            int bytesToRead = -1;
            while((bytesToRead = in.read(buffer)) != -1) {
                out.write(buffer,0, bytesToRead);
            }
        }catch (Exception ex){
            throw new ServiceException(ResultServiceEnums.DOWNLOAD_ERROR);
        }finally {
            if(in != null) {in.close();}
            if(out != null) {out.close();}
        }
    }



    @ApiImplicitParams({@ApiImplicitParam(name = "filePath", value = "文件路径", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "newFilePath", value = "新文件名（修改文件名时使用该参数）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "content", value = "内容（修改文件内容时使用该参数）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课Id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节Id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "fileType", value = "1新增文件夹，2新增文件，3修改文件名，4删除文件，5修改文件内容, 6读取文件内容", required = true, dataType = "varchar")})
    @ApiOperation(value="单文件或工程各种操作", notes="单文件或工程各种操作")
    @GetMapping(value = "/createOrUpdateFiles")
    public Result createOrUpdateFiles(String filePath, String newFilePath, String content, String slId, String chapId, String fileType) throws Exception{
        String codePath = "";
        String javaCode = "";
        codePath = global.getCOURSE_PROJECT_LOCATION();
        String serverPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapId + File.separator
                + codePath + File.separator
                + UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID) + File.separator;
        File file = new File(serverPath+filePath);
        String finished = bChapterService.queryIsCompleteByUserIdAndChapId(UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID),chapId);
        if(finished!=null&&finished.equals("1")&&!fileType.equals("6")){
            throw new ServiceException(ResultServiceEnums.EXPERIMENT_FINISHED);
        }
        if(fileType.equals("1")&&!file.exists()){//新增文件夹
            if(file.isFile()){
                throw new ServiceException(ResultServiceEnums.THIS_IS_FILE);
            }
            file.mkdirs();
        }else if(fileType.equals("2")&&!file.exists()){//新增文件
            if(file.isFile()){
                throw new ServiceException(ResultServiceEnums.THIS_IS_FILE);
            }
            file.createNewFile();
        }else if(fileType.equals("3")&&file.exists()){//修改文件名
            File newFile = new File(serverPath+newFilePath);
            file.renameTo(newFile);
        }else if(fileType.equals("4")&&file.exists()){//删除文件
            this.deleteFile(file);
        }else if(fileType.equals("5")&&file.exists()){//修改文件内容
            FileUtils.write(file,content,"UTF-8");
        }else if(fileType.equals("6")&&file.exists()){//读取文件内容
            javaCode = FileUtils.readFileToString(file);
        }
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                javaCode);
    }

    private void deleteFile(File file) {
        if (file.isFile()) {//判断是否是文件
            file.delete();//删除文件
        } else if (file.isDirectory()) {//否则如果它是一个目录
            File[] files = file.listFiles();//声明目录下所有的文件 files[];
            for (int i = 0;i < files.length;i ++) {//遍历目录下所有的文件
                this.deleteFile(files[i]);//把每个文件用这个方法进行迭代
            }
            file.delete();//删除文件夹
        }
    }


    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课Id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节Id", required = true, dataType = "varchar")})
    @ApiOperation(value="老师查看所有学生作业", notes="老师查看所有学生作业")
    @GetMapping(value = "/queryAllStudentCode")
    public Result queryAllStudentCode(String slId, String chapterId){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCodeServiceImpl.queryAllStudentCode(slId,chapterId));
    }


    @ApiImplicitParams({@ApiImplicitParam(name = "content", value = "代码", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "fileName", value = "class名", required = true, dataType = "varchar")})
    @ApiOperation(value="编译代码", notes="编译代码")
    @GetMapping(value = "/compileCode")
    public Result compileCode(String content, String studentCode, String fileName){
        String fileUrl = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getTEMP_LOCATION() + File.separator
                + global.getSTUDENT_CODE_LOCATION() + File.separator
                + studentCode + File.separator;
        String executeResult = "";
        try {
            executeResult = CompileUtil.executeJavaCode(content, fileName, fileUrl);

            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    executeResult);
        } catch (Exception e) {
            throw new ServiceException(ResultServiceEnums.COMPILING_CODE);
        }
    }


    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课Id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节Id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学号", required = true, dataType = "varchar")})
    @ApiOperation(value="获取智评分数", notes="获取智评分数")
    @GetMapping(value = "/getIntelligenceScore")
    public Result getIntelligenceScore(String slId, String chapterId, String studentCode){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCodeServiceImpl.getIntelligenceScore(slId,chapterId,studentCode));
    }



    @ApiOperation(value="老师查看学生的实验代码", notes="老师查看学生的实验代码")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "examType", value = "类型 0001:JAVA单文件，0002:JAVA工程，1001:HTML单文件，1002:HTML工程", required = true, dataType = "varchar")})
    @GetMapping(value = "/queryStudentExamFiles")
    public Result queryStudentExamFiles(@RequestParam("slId") String slId,
                                    @RequestParam("chapId") String chapId,
                                    @RequestParam("studentCode") String studentCode,
                                    @RequestParam("examType") String examType){
        List<BCodeFile> list = bCodeServiceImpl.queryStudentExamFiles(slId, chapId, examType,studentCode);
        if(examType.equals("0001")||examType.equals("1001"))
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    list.get(0));
        else
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    list);

    }


    @ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar")
    @ApiOperation(value="检查是否有学生提交作业", notes="检查是否有学生提交作业")
    @GetMapping(value = "/checkStudentFiles")
    public Result checkStudentFiles(String slId){
        int flag = bCodeServiceImpl.checkStudentFiles(slId);
        return Result.builder(new PersistModel(flag),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                "");
    }

}
