package org.rcisoft.business.bcode.service.impl;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.rcisoft.business.bcode.model.BCodeLxc;
import org.rcisoft.business.bcode.service.BCodeLxcService;
import org.rcisoft.business.bcode.task.StopLxcJob;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.command.LxcCommand;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.service.RcRedisService;
import org.rcisoft.core.service.SerializationUtils;
import org.rcisoft.core.util.FreemarkerUtil;
import org.rcisoft.core.util.TaskUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.*;
import java.util.logging.Logger;

/**
 * Created by lcy on 18/1/9.
 */
@Service
@Slf4j
public class BCodeLxcServiceImpl implements BCodeLxcService {


    /*任务组名*/
    private static final String JOB_LXC_GROUP = "lxc_job_group";
    /*触发器组名*/
    private static final String TRIGGER_LXC_GROUP = "lxc_trigger_group";


    @Autowired
    private RcRedisService rcRedisServiceImpl;

    @Autowired
    private Global global;

    @Autowired
    private FreemarkerUtil freemarkerUtil;

    @Autowired
    private SchedulerFactoryBean schedulerFactoryBean;

    @Autowired
    private LxcCommand lxcCommand;


    @Override
    public CommandResult startBCodeLxc(BCodeLxc lxc) {
        Logger logger = Logger.getAnonymousLogger();
        CommandResult result = null;
        List<String> ports  = null;
        /*初始化port 20000 - 30000 */
        Integer port = global.getLxcInitPort();
        Integer maxPort = global.getLxcMaxPort();
        Integer portSSH = global.getLxcInitPort()+10000;
        Integer maxPortSSH = global.getLxcMaxPort()+10000;
        ports = rcRedisServiceImpl.getList("lxcPorts");
        logger.info("------------------------------------------------------------------------------------------------");
        logger.info(ports.size()+"redis 端口使用数量");
        if(null == ports || 0 == ports.size())
            ports = new ArrayList<>();
        else{

            port = this.getIdleWithPorts(ports,port,maxPort);
            logger.info("<port>:"+port);
            portSSH = this.getIdleWithPorts(ports,portSSH,maxPortSSH);
            logger.info("<portSSH>:"+portSSH);
        }


        /*端口满了*/
        if(0 == port||0 == portSSH) {
            result = new CommandResult(ResultCode.FAIL,"端口全部占用,请稍后!",lxc);
            return result;
        }
        ports.add(port.toString());
        ports.add(portSSH.toString());
        /*1.freemarker 生成模板*/
        Map param = new HashMap<>();
        param.put("lxcName",lxc.getUserId() + "_" + lxc.getSl());
        param.put("lxcPort",port);
        param.put("lxcCode",lxc.getUserCode());
        param.put("lxcSshPort",portSSH);
        /* config path*/
        String destPath = global.getBASE_UPLOAD_SERVER_LOCATION() +
                 global.getLxcDockerfilePath() + "/" + lxc.getUserId()  + "/";
        /*启动路径*/
        String dockerPath = global.getPHYSICAL_UPLOAD_SERVER_LOCATION()  +
                global.getLxcDockerfilePath() + "/" + lxc.getUserId()  + "/";
        /*文件路径*/
        param.put("lxcPath",lxc.getWorkingDir());
        result = freemarkerUtil.generatorFileFromModel(this.getModelProject(lxc.getCode()),destPath
                ,param);
        if(!result.isSuccess()) {
            result = new CommandResult(ResultCode.FAIL,"容器配置文件有误!",lxc);
            return result;
        }

        /*2.docker-compose 起容器*/
        log.info("执行路径:{}",dockerPath);
        result = lxcCommand.startOrDownLxc(dockerPath);
        if(!result.isSuccess()) {
            result = new CommandResult(ResultCode.FAIL,"容器启动失败!",lxc);
            return result;
        }

        if(global.getJavaProject().equals(lxc.getCode()))
            port = portSSH;
        lxc.setContainerPort(port);
        lxc.setStartDate(new Date());
        lxc.setTotalMin(global.getLxcMinu());
        Date endTime = new Date(lxc.getStartDate().getTime() + ( global.getLxcMinu() * 60 * 1000 ));
        lxc.setShutdownDate(endTime);
        /*3.redis 放置状态*/
        String key = global.getLxcPrefix() + "_" + lxc.getUserId();
        rcRedisServiceImpl.setBytes(key,
                SerializationUtils.serializer(lxc));
        /*4.定时关闭 */
        schedulerFactoryBean.getScheduler();
        /*任务参数*/
        Map jobParam = new HashMap();
        jobParam.put("redisKeyId",key);
        jobParam.put("containerName",this.getModelProject(lxc.getCode()) + "_" + lxc.getUserId());
        jobParam.put("containerPath",dockerPath);
        jobParam.put("containerPort",port);
        JobDetail job = JobBuilder.newJob(StopLxcJob.class).
                withIdentity(global.getLxcPrefix() + lxc.getUserId(), JOB_LXC_GROUP).
                usingJobData(new JobDataMap(jobParam)).build();
        Trigger trigger = TriggerBuilder.newTrigger().forJob(job)
                .usingJobData(new JobDataMap(jobParam))
                .withIdentity(global.getLxcPrefix() + lxc.getUserId())
                .startAt(endTime).build();
                // 触发器时间设定
        /*触发器*/
        TaskUtil.startSchedule(schedulerFactoryBean.getScheduler(),trigger,job);
        /*5.端口放进去 */
        rcRedisServiceImpl.setList("lxcKeys",
                key);
        rcRedisServiceImpl.setList("lxcPorts",
                port.toString());
        return result;
    }

    /**
     * 获取空闲的端口  加锁 synchronized
     * @param ports
     * @param port
     * @param maxPort
     * @return
     */
    private synchronized Integer getIdleWithPorts(List<String> ports, Integer port, Integer maxPort) {
         Logger logger = Logger.getAnonymousLogger();
        logger.info("------------------------getIdleWithPorts-------------------------------------");
         logger.info("[portsSize:]"+ports.size());
        logger.info("[port jisuan:]"+(ports.size() -1  - (maxPort - port)) );
        int choosePort = 0;
        /*端口占用满了*/
        if(ports.size() -1  - (maxPort - port) == 0)
            return choosePort;
        for(int i = port ; i < maxPort ; i++){
            boolean flag = false;
            for (int j = 0 ; j < ports.size() ; j++){
                if(i - Integer.parseInt(ports.get(j)) == 0) {
                    flag = true;
                    break;
                }
            }
            if (!flag){
                choosePort = i;
                break;
            }

        }
        return choosePort;
    }

    /**
     * 或许模型
     * @param code
     * @return
     */
    private String getModelProject(String code) {
        if(global.getJavaProject().equals(code))
            return "java";
        else if(global.getHtmlProject().equals(code))
            return "apache";
        else
            return "";
    }


    @Override
    public CommandResult stopBCodeLxc(BCodeLxc lxc) {
        /*1.拼key*/
        String key = global.getLxcPrefix() + "_" + lxc.getUserId();
        /*2.拼docker-compose.yml */
        String destPath = global.getPHYSICAL_UPLOAD_SERVER_LOCATION() + File.separator +
                global.getLxcDockerfilePath() + File.separator + lxc.getUserId()  + File.separator;;
        CommandResult commandResult = null;

        commandResult = lxcCommand.startOrDownLxc(destPath,false);
        if(!commandResult.isSuccess())
            return commandResult;
        /*3.移除容器*/
        rcRedisServiceImpl.remove(key);
        /*4.移除端口*/
        rcRedisServiceImpl.removeList("lxcKeys",
                key);
        rcRedisServiceImpl.removeList("lxcPorts",lxc.getContainerPort() + "");
        return commandResult;
    }



    @Override
    public void clearCodeLxc(BCodeLxc lxc) {

    }


}
