package org.rcisoft.business.bcode.task;

import lombok.extern.slf4j.Slf4j;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.rcisoft.business.bcode.model.BCodeLxc;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.command.LxcCommand;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.service.RcRedisService;
import org.rcisoft.core.service.SerializationUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.Date;
import java.util.List;

/**
 * Created by lcy on 18/1/10.
 *
 *
 * 项目启动时,清除已到期的容器
 *             解决因 服务器down 而造成的 stopLxcJob 未执行问题
 *
 *
 */
@Component
@Slf4j
public class ClearLxcJob implements Job {

    @Autowired
    private RcRedisService rcRedisServiceImpl;

    @Autowired
    private LxcCommand lxcCommand;

    @Autowired
    private Global global;


    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        /*所有启动的容器*/
        List<String> keys = rcRedisServiceImpl.getList("lxcKeys");
        if(null == keys || 0 == keys.size())
            return;
        /*name*/
        for (String key :
                keys) {
            this.stopLxc(key);
        }
    }

    /**
     * 关闭容器
     * @param key
     */
    private void  stopLxc(String key) {
        String containerPath = null;
        byte[] results = rcRedisServiceImpl.getBytes(key);
        BCodeLxc lxc = null;
        if (null != results) {
            lxc = SerializationUtils.deserializer(results, BCodeLxc.class);
            if (null == lxc)
                return;
            Date shutdown = lxc.getShutdownDate();
            /*未到点*/
            if(null != shutdown && shutdown.after(new Date())) {
                /*todo 重新定时-------关闭*/

                return;
            }
            /*到点了*/
            containerPath = global.getPHYSICAL_UPLOAD_SERVER_LOCATION() + File.separator +
                    global.getLxcDockerfilePath() + File.separator + lxc.getUserId()  + File.separator;
            /*关闭容器*/
            /*2.docker-compose 起容器*/
            CommandResult commandResult = null;
            commandResult = lxcCommand.startOrDownLxc(containerPath,false);
            /*关闭失败 返回 不删除key*/
            if(!commandResult.isSuccess())
                return;
        }
        /*3.移除容器*/
        rcRedisServiceImpl.remove(key);
            /*4.移除端口*/
        rcRedisServiceImpl.removeList("lxcKeys",
                key);
        rcRedisServiceImpl.removeList("lxcPorts",lxc.getContainerPort() + "");
    }
}
