package org.rcisoft.business.beduclass.service.impl;

import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.beduclass.dao.BEduClassRepository;
import org.rcisoft.business.beduclass.entity.BEduClass;
import org.rcisoft.business.beduclass.service.BEduClassService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2018-4-17 14:33:36.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BEduClassServiceImpl implements BEduClassService {

    @Autowired
    private BEduClassRepository bEduClassRepository;


    /**
    * 保存 bEduClass
    * @param bEduClass
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BEduClass bEduClass){
        bEduClass.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(bEduClass);
        int line = bEduClassRepository.insertSelective(bEduClass);
        log.info(UserUtil.getUserInfoProp(bEduClass.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            bEduClass.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bEduClass
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BEduClass bEduClass){
        UserUtil.setCurrentMergeOperation(bEduClass);
        bEduClass.setDeleted();
        int line = bEduClassRepository.logicalDelete(bEduClass);
        log.info(UserUtil.getUserInfoProp(bEduClass.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        bEduClass.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 bEduClass
    * @param bEduClass
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BEduClass bEduClass){
        UserUtil.setCurrentMergeOperation(bEduClass);
        int line = bEduClassRepository.updateByPrimaryKeySelective(bEduClass);
        log.info(UserUtil.getUserInfoProp(bEduClass.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            bEduClass.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bEduClass
    * @param id
    * @return
    */
    public BEduClass findById(String id){
        return bEduClassRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bEduClass
    * @param bEduClass
    * @return
    */
    public List<BEduClass> findAllByPagination(PageUtil<BEduClass> paginationUtility, BEduClass bEduClass){
        bEduClass.setStart();
        bEduClass.setNotDeleted();
        return bEduClassRepository.queryBEduClasss(bEduClass);
    }

}
