package org.rcisoft.business.bfile.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.IOUtils;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.business.bfile.service.BFileService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.List;

/**
* Created by gaowenfneg on 2017-10-9 14:34:56.
*/
@RestController
@RequestMapping("/BVideo")
public class BFileController extends PaginationController<BFile> {
    @Autowired
    private BFileService bFileService;
    @Autowired
    private Global global;


    @ApiOperation(value="添加/编辑视频", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoName", value = "视频名称(1-200)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoUrl", value = "视频地址(1-200)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1002')")
    @PostMapping(value = "/add")
    public Result add(BFile bFile) {
        PersistModel data = bFileService.persist(bFile);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bFile);
    }

    @ApiOperation(value="逻辑删除", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bFileService.removeBVideo(id,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bFileService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询视频信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoName", value = "视频名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoUrl", value = "视频地址", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBVideosByPagination")
    public GridModel queryBVideosByPagination(BFile param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bFileService.queryBVideosByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询视频信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoName", value = "视频名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoUrl", value = "视频地址", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBVideos")
    public Result queryBVideos(BFile param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bFileService.queryBVideos(param));
    }


    @ApiOperation(value="上传文件", notes="上传文件")
    @ApiImplicitParams({@ApiImplicitParam(name = "chapterId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonId", value = "课程ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件,'2'为ppt,'3'为pdf", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "file", value = "视频文件", required = true, dataType = "MultipartFile")})
    @PreAuthorize("hasAnyRole('ROLE_1001','ROLE_1002')")
    @PostMapping(value = "/uploadVideo")
    public Result uploadVideo(BFile bFile, HttpServletRequest request) {
        List<MultipartFile> list = ((MultipartHttpServletRequest)request).getFiles("file");
            String videoUrl = bFileService.uploadVideoToServer(list, bFile,getToken());
            return Result.builder(new PersistModel(1),
                    "上传成功",
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    videoUrl);
    }

    @ApiOperation(value="上传license", notes="上传license证书文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "license文件", required = true, dataType = "MultipartFile")})
    @PostMapping(value = "/uploadLicenseWithoutAuth")
    public Result uploadLicense(HttpServletRequest request) {
        List<MultipartFile> list = ((MultipartHttpServletRequest)request).getFiles("file");
        try {
            if(list.size()>0){
                bFileService.uploadLicense(list.get(0));
            }else{
                throw new ServiceException(ResultServiceEnums.UPLOAD_ERROR);
            }

        } catch (IOException e) {
            e.printStackTrace();
            throw new ServiceException(ResultServiceEnums.UPLOAD_ERROR);
        }
        return Result.builder(new PersistModel(1),
                "license上传成功",
                MessageConstant.MESSAGE_ALERT_ERROR,
                "");
    }


    /**
     * 下载文件
     * @param chapterId
     * @param response
     * @return
     */
    @ApiOperation(value="下载文件", notes="下载文件")
    @ApiImplicitParam(name = "chapterId", value = "章节ID", required = true, dataType = "varchar")
    @PostMapping(value = "/downloadFile")
    public Result downloadFile(String chapterId, HttpServletResponse response){
        BFile video = bFileService.queryFileUrlByChapterId(chapterId);
        BufferedInputStream bis = null;
        BufferedOutputStream bos = null;
        if(video.getVideoUrl()!=""&&video.getVideoUrl()!=null){
            try {
                File file=new File(global.getBASE_UPLOAD_SERVER_LOCATION()+video.getVideoUrl());
                bis = new BufferedInputStream(new FileInputStream(file));
                bos = new BufferedOutputStream(response.getOutputStream());

                response.reset();
                response.setContentType("application/octet-stream;charset=utf-8");
                response.setHeader("Content-Disposition", "attachment;filename="+ new String((video.getVideoName()).getBytes(), "iso-8859-1"));
                response.setHeader("Access-Control-Allow-Origin", "*");
                IOUtils.copy(bis, bos);
            }catch (Exception e){
                e.printStackTrace();
            }finally {
                if (bis != null)
                    try {
                        bis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                if (bos != null)
                    try {
                        bos.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
            }
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    "");
        }else{
            return Result.builder(new PersistModel(0),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    "");
        }
    }
}

