package org.rcisoft.business.blesson.dao;

import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.entity.BLessonDirection;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * Created with CodeGenerator on .
 */
@Repository
public interface BLessonRepository extends BaseMapper<BLesson> {

    /**
     * 根据条件查找
     *
     * @param param
     * @return
     */
    @Select("<script>select b.*, bd.name as directionName, bd.business_id as directionId, bd.code as directionCode," +
            "IFNULL((SELECT SUM(bc.class_hour) from b_chapter bc where bc.lesson_id = b.business_id),0) as chapterClassHour " +
            "from b_lesson b " +
            "left join b_lesson_direction bld on bld.lession_id = b.business_id " +
            "left join b_direction bd on bd.business_id = bld.direction_id "+
            "where b.del_flag != 1 and b.flag = 1 " +
            "<if test=\"lessonName!=null and lessonName != ''\">and b.`lesson_name` LIKE CONCAT('%',#{lessonName},'%') </if>"+
            "<if test=\"directionId!=null and directionId != ''\">and bld.direction_id = #{directionId} </if>"+
            "<if test=\"code!=null and code != ''\">and b.`code` LIKE CONCAT('%',#{code},'%') </if></script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryBLessons(BLesson param);

    @Select("<script>select bl.*,bld.direction_id as directionId from b_lesson bl " +
            "left join b_lesson_direction bld on bl.business_id = bld.lession_id " +
            "where bl.code = #{code} and bl.del_flag != 1 and bl.flag = 1 </script>")
    @ResultMap(value = "BaseResultMap")
    BLesson queryBLessonByCode(String code);

    @Insert("<script>INSERT INTO b_lesson" +
            "(create_date,update_date,del_flag,flag,business_id,code,lesson_name,default_url,create_by,class_hour,credits)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.createDate},#{item.updateDate},#{item.delFlag},#{item.flag},#{item.businessId},#{item.code},#{item.lessonName},#{item.defaultUrl},#{item.createBy},#{item.classHour},#{item.credits})" +
            "</foreach></script>")
    int insertList(List<BLesson> bLessons);

    @Insert("<script>INSERT INTO b_lesson_direction (business_id,lession_id,direction_id,sl_id) VALUES " +
            "(#{businessId},#{lessionId},#{directionId},#{slId})</script>")
    int insertLessonDirection(BLessonDirection model);

    @Update("<script>update b_lesson_direction set direction_id = #{directionId} where lession_id = #{lessionId}</script>")
    int updateDirectionByLessonId(BLessonDirection direction);

    @Update("<script>update b_lesson_direction set direction_id = #{directionId} where sl_id = #{slId}</script>")
    int updateDirectionBySlId(BLessonDirection direction);

    @Select("SELECT COALESCE(SUM(class_hour),0) classHour FROM b_chapter where lesson_id = #{businessId} and pid != '-1'")
    int queryClassHourByLessonId(String businessId);
}

