package org.rcisoft.business.brclassstudent.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.collections.map.HashedMap;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.bclass.dao.BClassRepository;
import org.rcisoft.business.bclass.entity.BClass;
import org.rcisoft.business.brclassstudent.dao.BRClassStudentRepository;
import org.rcisoft.business.brclassstudent.entity.BRClassStudent;
import org.rcisoft.business.brclassstudent.service.BRClassStudentService;
import org.rcisoft.business.bstudent.dao.BStudentRepository;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BRClassStudentServiceImpl implements BRClassStudentService {
    @Autowired
    private BRClassStudentRepository bRClassStudentRepository;
    @Autowired
    private BClassRepository bClassRepository;
    @Autowired
    private BStudentRepository bStudentRepository;

    @Override
    public BRClassStudent selectOne(String businessId){
        return bRClassStudentRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BRClassStudent> queryBRClassStudentsByPagination(PageUtil pageUtil, BRClassStudent model) {
        Map param = new HashMap<String,Object>();
        return bRClassStudentRepository.queryBRClassStudents(param);
    }

    @Override
    public List<BRClassStudent> queryBRClassStudents(BRClassStudent model) {
        Map param = new HashMap<String,Object>();
        return bRClassStudentRepository.queryBRClassStudents(param);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BRClassStudent model){
        int line = 0;
        if(model.getBusinessId()!=null)
            line = bRClassStudentRepository.updateByPrimaryKeySelective(model);
        else {
            model.setCommonBusinessId();
           // UserUtils.setCurrentPersistOperation(model);
            line = bRClassStudentRepository.insertSelective(model);
        }
        //FamilyLogger.sysInfo(UserUtils.getUser().getBusinessId(),UserUtils.getUser().getLoginName()+"新增了ID为"+user.getBusinessId()+"的用户");
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel importExcel(HSSFWorkbook hwb) throws IOException {
        ArrayList<BRClassStudent> brClassStudents = new ArrayList<BRClassStudent>();
        ArrayList<String> studentCode = new ArrayList<String>();
        ArrayList<String> classCode = new ArrayList<String>();
        ArrayList<String> repeatCode = new ArrayList<String>();

        String[] headers = {"班级编号","学号"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers);  //获取excel数据

        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        for(String[] value:values){   //将数据封装到entity

            //1.0 检查信息是否为空
            BRClassStudent brClassStudent = null;
            if(value[0]==null||value[0].equals("")||value[1]==null||value[1].equals("")){
                continue;
            }

            //1.1 检查班级是否存在
            if((bClassRepository.selectOne(new BClass(value[0])))==null) {
                if(!classCode.contains(value[0]))
                classCode.add(value[0]);
                continue;
            }

            //1.2 检查学生是否存在
            if((bStudentRepository.selectOne(new BStudent(value[1])))==null) {
                if(!studentCode.contains(value[1]))
                studentCode.add(value[1]);
                continue;
            }

            //1.3 检查班级学生是否存在
            BRClassStudent classStudent = new BRClassStudent();
            //classStudent.setClassCode(value[0]);
            classStudent.setStudentCode(value[1]);
            if((bRClassStudentRepository.selectOne(classStudent))!=null) {
                repeatCode.add(value[1]);
                continue;
            }


            //1.4 构造学生班级信息
            brClassStudent= new BRClassStudent(value[0],value[1]);
            brClassStudent.setCommonBusinessId();
            //UserUtils.setCurrentPersistOperation(brClassStudent);

            if(!brClassStudents.contains(brClassStudent)){
                brClassStudents.add(brClassStudent);
            }
        }
        String result ="";
        if (brClassStudents.size()>0){
            int line = bRClassStudentRepository.insertList(brClassStudents);
            if (line>0){
              result  = "成功导入"+brClassStudents.size()+"条数据。";
            }else{
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }
        }

        StringBuilder nullResult = new StringBuilder();
        if(repeatCode.size()>0){
            //nullResult.append("以下学生班级关系已存在：").append(JSON.toJSONString(repeatCode)).append("。");
            nullResult.append("以下学生已存在于其他班级：").append(JSON.toJSONString(repeatCode)).append("。");
            //  nullResult.append("部分学生不存在，请检查后重新上传");
        }
        //2. 检查重复

        if(classCode.size()>0){
            nullResult.append("以下班级编号不存在：").append(JSON.toJSONString(classCode)).append("。");
          //  nullResult.append("部分班级不存在，请检查后重新上传");
        }
        if(studentCode.size()>0){
            nullResult.append("以下学生学号不存在：").append(JSON.toJSONString(studentCode)).append("。");
          //  nullResult.append("部分学生不存在，请检查后重新上传");
        }


        result+=nullResult;

        if(brClassStudents.size()<1){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),result);
        }
        //4.调用存储过程
        Map<String,Object> map = new HashedMap();
        map.put("result",null);
        bClassRepository.updateStudentNum(map);
        int l1 = MapUtils.getInteger(map,"result",-2);
        if(l1<=0)
            throw new ServiceException(ResultServiceEnums.UPDATE_STUDENT_NUM_ERROR);
        return new PersistModel(1,result.toString());
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBRClassStudents(String stuCode,
                                              String classCode) {
        BRClassStudent brClassStudent = bRClassStudentRepository.queryBRClassByCode(new BRClassStudent(classCode,stuCode));
        if(brClassStudent==null)
            throw new ServiceException(ResultServiceEnums.STUDENT_NOT_EXISTS);
        int line = bRClassStudentRepository.deleteByPrimaryKey(brClassStudent.getBusinessId());
        Map<String,Object> map = new HashedMap();
        map.put("result",null);
        bClassRepository.updateStudentNum(map);
        int l1 = MapUtils.getInteger(map,"result",-2);
        if(l1<=0)
            throw new ServiceException(ResultServiceEnums.UPDATE_STUDENT_NUM_ERROR);
        return new PersistModel(line);
    }


}
