package org.rcisoft.business.bteacher.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bteacher.entity.BTeacher;
import org.rcisoft.business.bteacher.service.BTeacherService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.UnsupportedEncodingException;
import java.util.List;

/**
* Created by CodeGenerator on 2017/07/12.
*/
@RestController
@RequestMapping("/BTeacher")
public class BTeacherController extends PaginationController<BTeacher> {
    @Autowired
    private BTeacherService bTeacherService;
    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑教师信息", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "教师工号(1-11)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "0:未推荐，1:推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐时间，排序依据", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因(1-200)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(BTeacher bTeacher, SysUser user) throws UnsupportedEncodingException {
        if(bTeacher.getCode()!=null&&bTeacher.getCode().length()>
                Integer.parseInt(global.getMAX_CODE_LENGTH()))
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
        user.setLoginName(bTeacher.getCode());
        PersistModel data = bTeacherService.persist(bTeacher,user,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTeacher);
    }

    @ApiOperation(value="逻辑删除", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bTeacherService.removeBTeacher(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }



    @ApiOperation(value="excel导入", notes="excel导入教师信息")
    @ApiImplicitParam(name = "importFile", value = "excel文件", required = true, dataType = "MultipartFile")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile) throws Exception {
        PersistModel data = bTeacherService.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getInfluenceReason());
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTeacherService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询教师信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "教师工号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "0:未推荐，1:推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐时间，排序依据", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBTeachersByPagination")
    public GridModel queryBTeachersByPagination(BTeacher param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bTeacherService.queryBTeachersByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询", notes="根据条件查询教师信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "教师工号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "0:未推荐，1:推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐时间，排序依据", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBTeachers")
    public Result queryBTeachers(BTeacher param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTeacherService.queryBTeachers(param));
    }

    @ApiOperation(value="查询推荐教师", notes="分页查询推荐教师")
    @GetMapping(value = "/queryTeacherWithRecommendWithoutAuth")
    public GridModel queryTeacherWithRecommend(){
        List<BTeacher> teachers = bTeacherService.queryTeachersWithRecommendByPagination(getPaginationUtility());
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }


}
