package org.rcisoft.business.bterm.service.impl;

import org.rcisoft.business.bclass.entity.BClass;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.business.bterm.enums.BTermEnum;
import org.rcisoft.business.bterm.service.BTermService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BTermServiceImpl implements BTermService {

    @Autowired
    private BTermRepository bTermRepository;
    @Autowired
    private BSlRepository bSlRepository;

    @Override
    public BTerm selectOne(String businessId){
        return bTermRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BTerm> queryBTermsByPagination(PageUtil pageUtil, BTerm model) {
        Map param = new HashMap<String,Object>();
        return bTermRepository.queryBTerms(param);
    }

    @Override
    public List<BTerm> queryBTerms(BTerm model) {
        Map param = new HashMap<String,Object>();
        return bTermRepository.queryBTerms(param);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel persist(BTerm model, String token){
        int line = 0;
        if(model.getBusinessId()!=null) {
            UserUtil.setCurrentMergeOperation(model);
            line = bTermRepository.updateByPrimaryKeySelective(model);
        }else {
            if(bTermRepository.selectOne(new BTerm(model.getCode()))!=null)
                throw new ServiceException(ResultServiceEnums.TERM_EXISTS);
            UserUtil.setCurrentPersistOperation(model);
            line = bTermRepository.insertSelective(model);
        }
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBTerm(String id) {
        if(bSlRepository.selectslNumByTermId(id)>0) {
            throw new ServiceException(ResultServiceEnums.TERM_HAS_SL);
        }
        int line = bTermRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel changeIS(String code, String token)
    {
        //这里没有直接使用  0  1 ，而是自定义了枚举类，来使用枚举类型
        BTerm bt=new BTerm();
        bt.setCode(code);
        List<BTerm> bTerm = bTermRepository.queryBtermByCode(code);
        //BTerm bTerm=bTermRepository.selectOne(bt);
        if(bTerm!=null)
        {
            if(bTerm.get(0).getIsStart().equals(BTermEnum.UN_START.getCode())) {
                bTerm.get(0).setIsStart(BTermEnum.START.getCode());
                //bTermRepository.updateUnStart(bTerm.get(0).getBusinessId());
            }
            else {
                bTerm.get(0).setIsStart(BTermEnum.UN_START.getCode());
            }
            //如果是用updateByPrimaryKey的话，那么在数据库中将没赋值的项 置空
            UserUtil.setCurrentMergeOperation(bTerm.get(0));
            int line = bTermRepository.updateByPrimaryKeySelective(bTerm.get(0));
            return new PersistModel(line);
        }
        else
        {
            throw new ServiceException(ResultServiceEnums.UPDATE_PASSWORD_ERROR);
        }
    }

    @Override
    public List<SelectModel> queryBTermList() {
        Date date = new Date();
        String nowTime = new SimpleDateFormat("yyyy-MM-dd").format(date);
        List<BTerm> bTerm = bTermRepository.queryBTermList(nowTime);
        ArrayList<SelectModel> comBoxModel = new ArrayList<SelectModel>();
        for(BTerm c : bTerm){
            SelectModel selectModel = new SelectModel();
            selectModel.setId(c.getCode());
            selectModel.setText(c.getName());
            comBoxModel.add(selectModel);
        }
        return comBoxModel;
    }

}
