package org.rcisoft.business.texaminfo.service.impl;

import net.sf.json.JSONArray;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bchapter.service.impl.BChapterServiceImpl;
import org.rcisoft.business.texaminfo.OptionsDto;
import org.rcisoft.business.texaminfo.dao.TExamDetailRepository;
import org.rcisoft.business.texaminfo.entity.TExamDetail;
import org.rcisoft.business.tquestion.entity.TQuestion;
import org.rcisoft.business.tquestion.entity.TQuestionOptions;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.service.RcRedisService;
import org.rcisoft.core.service.SerializationUtils;
import org.rcisoft.core.util.IpUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.texaminfo.dao.TExamInfoRepository;
import org.rcisoft.business.texaminfo.entity.TExamInfo;
import org.rcisoft.business.texaminfo.service.TExamInfoService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.ArrayList;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by yangzhaojun on 2018-1-15 19:49:07.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TExamInfoServiceImpl implements TExamInfoService {

    @Autowired
    private TExamInfoRepository tExamInfoRepository;

    @Autowired
    private TExamDetailRepository tExamDetailRepository;

    @Autowired
    private BChapterService bChapterService;

    @Autowired
    private RcRedisService rcRedisServiceImpl;

    /**
    * 保存 tExamInfo
    * @param chapterId
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(String chapterId, String userId,TExamInfo tExamInfo, HttpServletRequest request){
        List<String> list = tExamInfoRepository.queryPidByChapterId(chapterId);
        if(list.size()>0){
            String pId = list.get(0);
            tExamInfo.setPId(pId);
            tExamInfo.setUId(userId);
            tExamInfo.setIp(IpUtil.getIpAddr(request));
            int line = tExamInfoRepository.insertSelective(tExamInfo);
            tExamInfoRepository.updateStatus(pId);
            return new PersistModel(line,pId);
        }else{
            throw new ServiceException(ResultServiceEnums.NO_PAPER_EXISTS);
        }

    }


    /**
    * 逻辑删除
    * @param tExamInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(TExamInfo tExamInfo){
        UserUtil.setCurrentMergeOperation(tExamInfo);
        int line = tExamInfoRepository.logicalDelete(tExamInfo);
        log.info(UserUtil.getUserInfoProp(tExamInfo.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        tExamInfo.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 tExamInfo
    * @param tExamInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TExamInfo tExamInfo){
        UserUtil.setCurrentMergeOperation(tExamInfo);
        int line = tExamInfoRepository.updateByPrimaryKeySelective(tExamInfo);
        log.info(UserUtil.getUserInfoProp(tExamInfo.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            tExamInfo.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 tExamInfo
    * @param id
    * @return
    */
    public TExamInfo findById(String id){
        return tExamInfoRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 tExamInfo
    * @param tExamInfo
    * @return
    */
    public List<TExamInfo> findAllByPagination(PageUtil<TExamInfo> paginationUtility,
        TExamInfo tExamInfo){
        return tExamInfoRepository.queryTExamInfos(tExamInfo);
    }

    @Override
    public PersistModel finishPractice(OptionsDto options, String userId) {
        String paperId = tExamInfoRepository.queryPaperIdById(options.getExamInfoId());
        List<TExamDetail> examDetails = new ArrayList<>();
        //遍历单选
        JSONArray choiceJson = JSONArray.fromObject(options.getChoiceOptions());
        List<TQuestion> choiceOptions= (List<TQuestion>)JSONArray.toCollection(choiceJson, TQuestion.class);
        JSONArray multiSelectJson = JSONArray.fromObject(options.getMultiSelectOptions());
        List<TQuestion> multiSelectOptions= (List<TQuestion>)JSONArray.toCollection(multiSelectJson, TQuestion.class);
        JSONArray judgmentJson = JSONArray.fromObject(options.getJudgmentOptions());
        List<TQuestion> judgmentOptions= (List<TQuestion>)JSONArray.toCollection(judgmentJson, TQuestion.class);

        for(TQuestion choiceOption : choiceOptions){
            TExamDetail examDetail = new TExamDetail();
            examDetail.setPId(paperId);
            examDetail.setEId(options.getExamInfoId());
            examDetail.setUId(userId);
            examDetail.setQId(choiceOption.getBusinessId());
            examDetail.setUAnswer(choiceOption.getUanswer());
            examDetail.setQtype("1");
            examDetails.add(examDetail);
        }
        //遍历多选
        for(TQuestion multiSelectOption : multiSelectOptions){
            TExamDetail examDetail = new TExamDetail();
            examDetail.setPId(paperId);
            examDetail.setEId(options.getExamInfoId());
            examDetail.setUId(userId);
            examDetail.setQId(multiSelectOption.getBusinessId());
            examDetail.setUAnswer(multiSelectOption.getUanswer());
            examDetail.setQtype("2");
            examDetails.add(examDetail);
        }
        //遍历判断
        for(TQuestion judgmentOption : judgmentOptions){
            TExamDetail examDetail = new TExamDetail();
            examDetail.setPId(paperId);
            examDetail.setEId(options.getExamInfoId());
            examDetail.setUId(userId);
            examDetail.setQId(judgmentOption.getBusinessId());
            examDetail.setUAnswer(judgmentOption.getUanswer());
            examDetail.setQtype("3");
            examDetails.add(examDetail);
        }
        int result = tExamDetailRepository.insertList(examDetails);
        //清除缓存
        rcRedisServiceImpl.remove(userId + "_" + options.getChapterId());
        //更改练习状态为完成
        bChapterService.updateToFinish("4",options.getStudentCode(),options.getChapterId(),options.getSlId());
        return new PersistModel(result);
    }

}
