package org.rcisoft.business.tquestion.service.impl;

import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bstudent.dao.BStudentRepository;
import org.rcisoft.business.texaminfo.dao.TExamDetailRepository;
import org.rcisoft.business.texaminfo.dao.TExamInfoRepository;
import org.rcisoft.business.texaminfo.entity.TExamInfo;
import org.rcisoft.business.texaminfo.service.TExamInfoService;
import org.rcisoft.business.tpaperchapter.dao.TPaperChapterRepository;
import org.rcisoft.business.tquestion.dto.QuestionListAndNumDto;
import org.rcisoft.business.tquestion.dao.TQuestionOptionsRepository;
import org.rcisoft.business.tquestion.entity.TQuestionOptions;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.service.RcRedisService;
import org.rcisoft.core.service.SerializationUtils;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.tquestion.dao.TQuestionRepository;
import org.rcisoft.business.tquestion.entity.TQuestion;
import org.rcisoft.business.tquestion.service.TQuestionService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;

import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by yangzhaojun on 2018-1-15 19:49:07.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TQuestionServiceImpl implements TQuestionService {

    @Autowired
    private TQuestionRepository tQuestionRepository;

    @Autowired
    private TQuestionOptionsRepository tQuestionOptionsRepository;

    @Autowired
    private TExamInfoRepository tExamInfoRepository;

    @Autowired
    private TExamDetailRepository tExamDetailRepository;

    @Autowired
    private TExamInfoService tExamInfoService;

    @Autowired
    private BStudentRepository bStudentRepository;

    @Autowired
    private BChapterRepository bChapterRepository;

    @Autowired
    private TPaperChapterRepository tPaperChapterRepository;

    @Autowired
    private RcRedisService rcRedisServiceImpl;

    @Autowired
    private Global global;

    /**
    * 保存 tQuestion
    * @param tQuestion
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(TQuestion tQuestion){
        //增加主表题干信息
        UserUtil.setCurrentPersistOperation(tQuestion);
        int line = tQuestionRepository.insertSelective(tQuestion);
        //增加选项信息
        List<TQuestionOptions> options = new ArrayList<>();
        if(!tQuestion.getQtype().equals("3")){
            int i = 0;
            for (String option : tQuestion.getQuestionOptions().split(",")){
                TQuestionOptions TOptions = new TQuestionOptions();
                TOptions.setCommonBusinessId();
                TOptions.setQid(tQuestion.getBusinessId());
                TOptions.setAlias(String.valueOf(i));
                TOptions.setRepresent(option);
                options.add(TOptions);
                i++;
            }
            tQuestionRepository.insertOptionsList(options);
        }
        return new PersistModel(line);
    }


    /**
    * 逻辑删除
    * @param id
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int remove(String id, String token){
        TQuestion tQuestion = new TQuestion();
        tQuestion.setBusinessId(id);
        UserUtil.setCurrentMergeOperation(tQuestion);
        tQuestion.setDeleted();
        int line = tQuestionRepository.logicalDelete(tQuestion);
        /*log.info(UserUtil.getUserInfoProp(token,UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
            tQuestion.getBusinessId()+"的信息");*/
        return line;
    }

    /**
    * 修改 tQuestion
    * @param tQuestion
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TQuestion tQuestion){
        UserUtil.setCurrentMergeOperation(tQuestion);
        int line = tQuestionRepository.updateByPrimaryKeySelective(tQuestion);
        //修改选项信息
        List<TQuestionOptions> options = new ArrayList<>();
        if(!tQuestion.getQtype().equals("3")){
            tQuestionOptionsRepository.deleteByQId(tQuestion.getBusinessId());
            int i = 0;
            for (String option : tQuestion.getQuestionOptions().split(",")){
                TQuestionOptions TOptions = new TQuestionOptions();
                TOptions.setCommonBusinessId();
                TOptions.setQid(tQuestion.getBusinessId());
                TOptions.setAlias(String.valueOf(i));
                TOptions.setRepresent(option);
                options.add(TOptions);
                i++;
            }
            tQuestionRepository.insertOptionsList(options);
        }
        return new PersistModel(line);
    }

    /**
    * 根据id查询 tQuestion
    * @param id
    * @return
    */
    public TQuestion findById(String id){
        return tQuestionRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 tQuestion
    * @param tQuestion
    * @return
    */
    public List<TQuestion> findAllByPagination(PageUtil<TQuestion> paginationUtility,TQuestion tQuestion){
        tQuestion.setStart();
        tQuestion.setNotDeleted();
        return tQuestionRepository.queryTQuestions(tQuestion);
    }

    @Override
    public List<TQuestion> queryTQuestions(TQuestion tQuestion) {
        tQuestion.setStart();
        tQuestion.setNotDeleted();
        //已添加到本节的习题
        List<TQuestion> chapterQuestions = tQuestionRepository.queryChapterQuestionsByChapterId(tQuestion.getChapId());

        if(tQuestion.getPcompose().equals("1")){
            tQuestion.setChapId("");
        }
        //全部习题
        List<TQuestion> questions = tQuestionRepository.queryTQuestions(tQuestion);
        if(questions.size()>0){
            for(TQuestion question : questions) {
                if(!question.getQtype().equals("3")){
                    //不是判断题    获取选项
                    List<TQuestionOptions> options = tQuestionOptionsRepository.queryTQuestionOptions(question.getBusinessId());
                    question.setOptions(options);
                }
                if(tQuestion.getStateBeforeUpdate().equals("2")&&(tQuestion.getComposeBeforeUpdate().equals("1"))){
                    for(TQuestion chapterQuestion : chapterQuestions){
                        //判断是否已被添加
                        if(question.getBusinessId().equals(chapterQuestion.getBusinessId())){
                            question.setInUse("1");
                        }
                    }
                }

            }
        }

        return questions;
    }

    @Override
    public int removeByIds(String idInfoList) {
        return tQuestionRepository.deleteByIds(idInfoList);
    }

    @Override
    public TQuestion queryTQestionById(String businessId) {
        TQuestion tQuestion = tQuestionRepository.selectOne(new TQuestion(businessId));
        if(!tQuestion.getQtype().equals("3")){
            //选择题    获取选项
            List<TQuestionOptions> options = tQuestionOptionsRepository.queryTQuestionOptions(tQuestion.getBusinessId());
            tQuestion.setOptions(options);
        }
        return tQuestion;
    }

    @Override
    public QuestionListAndNumDto queryQuestionsByChap_IdAndQtype(String chapterId, String userId, HttpServletRequest request) {
        //从缓存中获取数据
        byte[] results = rcRedisServiceImpl.getBytes(userId + "_" + chapterId);
        QuestionListAndNumDto questionListAndNum = new QuestionListAndNumDto();
        //判断缓存中是否有数据，有数据直接返回；没有数据先查询数据，再放入缓存
        if(results!=null){
            questionListAndNum = SerializationUtils.deserializer(results, QuestionListAndNumDto.class);
            if(questionListAndNum!=null)
                return questionListAndNum;
        }
        String pId = "";
        TExamInfo tExamInfo = tExamInfoRepository.queryIdByUserId(userId,chapterId);
        List<String> explainResult = tPaperChapterRepository.queryDescByChapId(chapterId);
        String explain = explainResult.size()>0?explainResult.get(0):"";
        if(tExamInfo==null){
            //占卷子
            tExamInfo = new TExamInfo();
            UserUtil.setCurrentPersistOperation(tExamInfo);
            pId = String.valueOf(tExamInfoService.save(chapterId, userId,tExamInfo,request).getResult());
        }else{
            pId = tExamInfo.getPId();
        }
        String stuId = bStudentRepository.queryStudentByUserId(userId);
        //判断是否已做过练习
        String isFinish = bChapterRepository.queryPaperFinish(chapterId,stuId);
        List<TQuestion> choiceList = new ArrayList<>();
        List<TQuestion> multiSelectList = new ArrayList<>();
        List<TQuestion> judgmentList = new ArrayList<>();
        List<TQuestion> questionList = tQuestionRepository.queryQuestionsByChap_IdAndQtype(pId);
        for(TQuestion question : questionList){
            if(question==null){
                throw new ServiceException(ResultServiceEnums.QUESTION_ERROR);
            }
            if(question.getQtype().equals("1")){
                //获取全部选项
                List<TQuestionOptions> options = tQuestionOptionsRepository.queryTQuestionOptions(question.getBusinessId());
                question.setOptions(options);
                if(isFinish!=null&&isFinish.equals("1")){
                    //获取学生选择的选项
                    String uAnswer = tExamDetailRepository.queryUAnswer(question.getBusinessId(),userId,tExamInfo.getBusinessId());
                    if(uAnswer!=null){
                        question.setUanswer(uAnswer);
                    }
                }
                choiceList.add(question);
            }else if(question.getQtype().equals("2")){
                //获取全部选项
                List<TQuestionOptions> options = tQuestionOptionsRepository.queryTQuestionOptions(question.getBusinessId());
                question.setOptions(options);
                if(isFinish!=null&&isFinish.equals("1")){
                    //获取学生选择的选项
                    String uAnswer = tExamDetailRepository.queryUAnswer(question.getBusinessId(),userId,tExamInfo.getBusinessId());
                    if(uAnswer!=null){
                        question.setUanswer(uAnswer);
                    }
                }
                multiSelectList.add(question);
            }else {
                if(isFinish!=null&&isFinish.equals("1")){
                    String uAnswer = tExamDetailRepository.queryUAnswer(question.getBusinessId(),userId,tExamInfo.getBusinessId());
                    question.setUanswer(uAnswer);
                }
                judgmentList.add(question);
            }
        }
        questionListAndNum.setChoiceList(choiceList);
        questionListAndNum.setMultiSelectList(multiSelectList);
        questionListAndNum.setJudgmentList(judgmentList);
        questionListAndNum.setExamInfoId(tExamInfo.getBusinessId());
        questionListAndNum.setExplain(explain);
        questionListAndNum.setIsFinish(isFinish);

        questionListAndNum.setNumOfChoice(tQuestionRepository.queryNumByType("1",chapterId).get(0));
        questionListAndNum.setNumOfMultiSelect(tQuestionRepository.queryNumByType("2",chapterId).get(0));
        questionListAndNum.setNumOfJudgment(tQuestionRepository.queryNumByType("3",chapterId).get(0));
        //将数据放入缓存中
        rcRedisServiceImpl.setBytes(userId + "_" + chapterId,
                                    SerializationUtils.serializer(questionListAndNum),
                                    global.getREDIS_TIMEOUT());
        return questionListAndNum;
    }

    @Override
    public List<TQuestion> queryTQuestionBySlId(TQuestion tQuestion) {
        tQuestion.setStart();
        tQuestion.setNotDeleted();
        List<TQuestion> questions = tQuestionRepository.queryTQuestions(tQuestion);
        if(questions.size()>0){
            for(TQuestion question : questions) {
                if(!question.getQtype().equals("3")){
                    //不是判断题    获取选项
                    List<TQuestionOptions> options = tQuestionOptionsRepository.queryTQuestionOptions(question.getBusinessId());
                    question.setOptions(options);
                }
            }
        }

        return questions;
    }

}
