package org.rcisoft.core.command;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.result.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;

/**
 * Created by lcy on 18/1/9.
 *
 * 开启容器 命令
 */
@Slf4j
@Component
public class LxcCommand {

    @Autowired
    private Global global;

    @Autowired
    private RemoteExecuteCommand remoteExecuteCommand;

    /**
     * 开启容器
     */
    protected static final String START_LXC_COMMAND = "docker-compose up -d";
    /**
     * 关闭容器
     */
    protected static final String DOWN_LXC_COMMAND = "docker-compose down";
    /**
     * 配置文件
     */
    public static final String LXC_SETTING_NAME = "docker-compose.yml";
    /**
     * 子配置文件（Java目录下）
     */
    public static final String LXC_JAVA_NAME = "Dockerfile";

    public  CommandResult startOrDownLxc(String dest){
        return this.startOrDownLxc(dest,true);
    }


    public  CommandResult startOrDownLxc(String dest,boolean isStart){
        if("1".equals(global.getLxcIsRemote()))
            return this.startOrDownLxc(dest,isStart,true);
        else
            return this.startOrDownLxc(dest,isStart,false);
    }

    /**
     * 开/关容器
     * @param dest
     * @param isStart
     */
    public CommandResult startOrDownLxc(String dest,boolean isStart,boolean isRemote){
        CommandResult commandResult = null;
        try {
            File dirPath = new File(dest);
            if(!isRemote) {
                if (!dirPath.exists() || !dirPath.isDirectory()) {
                    log.error("目录错误 ...." + dest);
                    commandResult = new CommandResult(ResultCode.FAIL, "目录不存在", null);
                    return commandResult;
                }
            }
            if (isStart)
                if (isRemote)
                    remoteExecuteCommand.execute(START_LXC_COMMAND,dest);
                else
                Runtime.getRuntime().exec("cd " + dest + " && " + START_LXC_COMMAND);
            else{
                if (isRemote)
                    remoteExecuteCommand.execute(DOWN_LXC_COMMAND,dest);
                else
                    Runtime.getRuntime().exec("cd " + dest + " && " + DOWN_LXC_COMMAND);
                if(!isRemote) {
                    dirPath.delete();
                }
            }
            commandResult = new CommandResult(ResultCode.SUCCESS,null,null);
        }catch (Exception e){
            log.error(e.getMessage());
            commandResult = new CommandResult(ResultCode.FAIL,e.getMessage(),null);
        }
        return commandResult;

    }
}
