package org.rcisoft.core.util;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by gaowenfeng on 2017/7/12.
 * excel导入导出工具类
 */
public class ExcelUtil {
    static DecimalFormat df = new DecimalFormat("0");  //动态转换

    /**
     * 获取excel模板信息
     * @param hwb 源数据
     * @param headers  excel模板头
     * @return  整理好的ArrayList<String[]>
     */
    public static ArrayList<String[]> importExcel(HSSFWorkbook hwb, String[] headers){
        ArrayList<String[]> values = new ArrayList<>();
        for (int i = 0; i < hwb.getNumberOfSheets(); i++) {
            HSSFSheet hs = hwb.getSheetAt(i);
            if (hs == null)
                continue;
            else {
                int firstrownum = hs.getFirstRowNum();  //第一行
                int lastrownum = hs.getLastRowNum();    //最后一行
                List<Integer> emptyColList=new ArrayList<>();
                for (int j = firstrownum + 1; j <= lastrownum; j++) {
                    HSSFRow hr = hs.getRow(j);
                    if(hr==null||hr.getLastCellNum()<0){
                        continue;
                    }

                    int firstcolumnnum = hr.getFirstCellNum();
                    int lastcolumnnum = hr.getLastCellNum();
                    String[] value = new String[lastcolumnnum];
                    boolean hasEmpty = false;
                    for (int k = firstcolumnnum; k < lastcolumnnum; k++) {
                        HSSFCell hc = hr.getCell(k);
                        if (j==firstrownum + 1) {
                            //判断表头那一列为空数据并记录。
                            if(parseDB(hc).trim().equals("")) {
                                emptyColList.add(-1);
                            }else{
                                emptyColList.add(k);
                            }
                        }
                        else {
                            if (parseDB(hc).trim().equals("")&&emptyColList.get(k)!=-1) {
                                //在第j行第k列有空数据，且该列表头不是空的，则舍弃本行数据。
                                hasEmpty = true;

                                break;
                            }
                        }

                        value[k] = parseDB(hc);
                                //parseDB(hc.getCellType(), hc);
                    }
                    if(j==firstrownum + 1){
                        for(int h=0;h<headers.length;h++){
                            if(!value[h].trim().equals(headers[h]))
                                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_HEADER_ERROR);
                        }
                    }else{
                        if (!hasEmpty) {
                            values.add(value);
                        }
                    }
                }
            }
        }
        return values;
    }


    /**
     * 重复数据检查
     * @param repeatCode
     * @return
     */
    public static String checkRepeat(List<String> repeatCode){
        StringBuilder repeatMessage = new StringBuilder();
        if(repeatCode.size()>0){
            repeatMessage.append("以下编号的数据导入重复:");
            for(int i=0;i<repeatCode.size();i++){
                if(!repeatCode.get(i).equals("")&&repeatCode.get(i)!=null) {
                    if (i == 0)
                        repeatMessage.append(repeatCode.get(i));
                    else
                        repeatMessage.append(",").append(repeatCode.get(i));
                }
            }
        }else {
            repeatMessage.append("全部数据已经导入");
        }
        return repeatMessage.toString();
    }

    /**
     * 准备工作
     * @param hc
     * @return
     */
    private  static String parseDB(HSSFCell hc) {
        CellType celltype = hc.getCellTypeEnum();
        String cellValue=null;
        if (celltype == CellType.STRING) {
            cellValue = String.valueOf(hc.getStringCellValue());
        }
        else if (celltype == CellType.NUMERIC) {
            cellValue = String.valueOf(df.format(hc.getNumericCellValue()));
        }else if(celltype == CellType.BOOLEAN){
            cellValue = String.valueOf(hc.getBooleanCellValue());
        }else if(celltype == CellType.BLANK) {
            cellValue = hc.getStringCellValue();
        }else if(celltype == CellType.ERROR)
            cellValue = "";
        return cellValue;
    }

}
