package org.rcisoft.sys.role.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.List;

/**
 * Created by lcy on 16/7/30.
 */
@RestController
@RequestMapping("/role")
public class SysRoleController extends PaginationController<SysRole> {

    @Autowired
    private SysRoleService sysRoleServiceImpl;

     @ApiOperation(value="查询角色名是否合法", notes="查询角色名是否合法（true 合法   false 不合法  不能有其它的输出）")
     @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
             @ApiImplicitParam(name = "name", value = "角色名称(1-50)", required = false, dataType = "varchar"),
             @ApiImplicitParam(name = "code", value = "权限code(1-50)", required = false, dataType = "varchar"),
             @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "queryRoleValidate")
    public String queryRoleValidate(SysRole sysRole){
    	List<SysRole> sysRoleList = sysRoleServiceImpl.queryRolesByPagination(getPaginationUtility(), sysRole);
    	if(sysRoleList.size()==0)
    		return "true";
    	else 
    		return "false";
    }

    @ApiOperation(value="分页查询角色", notes="分页查询角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "角色名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "权限code", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "queryRoles")
    public GridModel queryRoles(SysRole sysRole){
        sysRoleServiceImpl.queryRolesByPagination(getPaginationUtility(), sysRole);
        GridModel gridModel = getGridModelResponse();
        return  gridModel;
    }


    @ApiOperation(value="分页查询角色", notes="根据userId 进行选中checked")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "userId", value = "用户ID", required = false, dataType = "varchar", paramType = "path"),
            @ApiImplicitParam(name = "name", value = "角色名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "权限code", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "queryRolesByUser/{userId}")
    public GridModel queryRolesByUser(@PathVariable("userId") String userId, SysRole sysRole){
        sysRoleServiceImpl.queryUserRolesByPagination(getPaginationUtility(),userId, sysRole);
        GridModel gridModel = getGridModelResponse();
        return  gridModel;
    }

    
    @ApiOperation(value="增加角色", notes="增加角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "userId", value = "用户ID", required = false, dataType = "varchar", paramType = "path"),
            @ApiImplicitParam(name = "name", value = "角色名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "权限code", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "adminLog/roleAdd")
    public Result roleAdd(@Valid SysRole sysRole, BindingResult br){
        PersistModel persistModel = sysRoleServiceImpl.persistRole(sysRole,getToken());
        if(persistModel.isSuccessBySinglePersist())
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    sysRole);

        else
            return Result.builder(new PersistModel(0),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    sysRole);
    }

    @ApiOperation(value="to 修改用户页面", notes="to 修改用户页面")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar", paramType = "path")
    @PostMapping(value = "toBg/toRoleUpdatePage/{id}")
    public ModelAndView toUserUpdatePage(@PathVariable("id") String id, ModelAndView modelAndView){
        modelAndView.getModelMap().addAttribute("roleUpdate", sysRoleServiceImpl.queryRoleById(id));
        modelAndView.setViewName("background/roleAdd");
        return modelAndView;
    }
    
    @ApiOperation(value="修改角色", notes="修改角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "userId", value = "用户ID", required = false, dataType = "varchar", paramType = "path"),
            @ApiImplicitParam(name = "name", value = "角色名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "权限code", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "adminLog/roleUpdate")
    public Result roleUpdate(@Valid SysRole sysRole, BindingResult br){
        PersistModel persistModel = sysRoleServiceImpl.mergeRole(sysRole,getToken());
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                null);
    }

    @ApiOperation(value="删除角色", notes="to 删除角色")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar", paramType = "path")
    @PostMapping(value = "adminLog/roleDelete/{id}")
    public Result roleDelete(@PathVariable("id") String id){

        PersistModel persistModel = sysRoleServiceImpl.removeRole(id,getToken());
        if(persistModel.isSuccessBySinglePersist())
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    id);
        else
            return Result.builder(new PersistModel(0),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    id);
    }

    @ApiOperation(value="权限分配", notes="权限分配")
    @ApiImplicitParams({@ApiImplicitParam(name = "roleId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "role_menus", value = "角色绑定权限", required = false, dataType = "varchar")})
    @PostMapping(value = "adminLog/rolePermission")
    public Result rolePermission(@RequestParam("roleId") String roleId, @RequestParam("role_menus") String role_menus ){

        return Result.builder(sysRoleServiceImpl.doAuthorization(roleId,role_menus,getToken()),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                roleId);
    }
    
    @ApiOperation(value="岗位绑定", notes="岗位绑定")
    @ApiImplicitParams({@ApiImplicitParam(name = "roleId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "role_depts", value = "角色部门绑定", required = false, dataType = "varchar")})
    @PostMapping(value = "adminLog/deptPermission")
    public Result deptPermission(@RequestParam("roleId") String roleId, @RequestParam("role_depts") String role_depts){
        PersistModel persistModel = sysRoleServiceImpl.doAuthorizationDept(roleId,role_depts,getToken());
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                roleId);
    }
}
