package org.rcisoft.sys.user.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
/**
 * Created by Andrew on 2017/7/6.
 */
@RestController
@RequestMapping(value = "/user")
public class SysUserController extends PaginationController<SysUser> {

    @Autowired
    private SysUserService userServiceImpl;

    @Autowired
    private Global global;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @PostMapping(value = "/addUser")
    public Result addUser(SysUser user, Result result){
        try{
            userServiceImpl.addUser(user);
            //response.setErrorMessage("添加失败",null);
        }catch (Exception e){
            e.printStackTrace();
            //response.setErrorMessage(e.getMessage(),"");
        }
        return result;
    }


    @ApiOperation(value="修改密码", notes="根据用户ID修改密码")
    @ApiImplicitParams({@ApiImplicitParam(name = "oldPassword", value = "旧密码", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "newPassword", value = "新密码", required = true, dataType = "varchar")})
    @PostMapping(value = "/changePassword")
    public Result changePassword(String oldPassword, String newPassword){
        int min_length = Integer.parseInt(global.getPASSWORD_MIN_PASSWORD());
        int max_length = Integer.parseInt(global.getPASSWORD_MAX_PASSWORD());
        int this_length = newPassword.length();
        if(this_length<min_length||this_length>max_length) {
            throw new ServiceException(ResultServiceEnums.PASSWORD_LENGTH_ERROR);
        }
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        String encodePassword = userServiceImpl.checkPassword(userId);
        boolean real =passwordEncoder.matches(oldPassword,encodePassword);
        if(!real) {
            throw new ServiceException(ResultServiceEnums.PASSWORD_ERROR);
        }
        PersistModel persistModel = new PersistModel(userServiceImpl.changePassword(userId,newPassword));
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                "");
    }

    @ApiOperation(value = "重置密码", notes = "根据用户学号或者工号重置密码")
    @PostMapping("/resetPassword")
    @ApiImplicitParam(name = "userCode", value = "学号或工号", required = true, dataType = "varchar")
    public Result resetPassword (String userCode){
        PersistModel persistModel = new PersistModel(userServiceImpl.resetPassword(userCode));
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userCode);
    }
}
