package org.rcisoft.sys.user.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.google.common.collect.Lists;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;
import org.rcisoft.core.entity.IdEntity;
import org.rcisoft.sys.role.entity.SysRole;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.util.StringUtils;

import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by lcy on 17/11/21.
 */
@Entity
@Table(name = "s_user")
@Data
@NoArgsConstructor
@AllArgsConstructor
public class SysUser extends IdEntity<SysUser> {

    private static final long serialVersionUID = 8885558381185320493L;


    /** 密码 */
    @Length(min = 1,max = 100,message = "长度最小为1,最大为80")
    @NotBlank
    private String password;
    /** 用户名 */
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    @NotBlank
    private String loginName;
    /** 用户名 */
    @Length(min = 1,max = 15,message = "长度最小为1,最大为10")
    @NotBlank
    private String name;
    @Length(min = 1,max = 20,message = "长度最小为1,最大为15")
    @NotBlank
    private String email;	// 邮箱
    @Length(min = 1,max = 11,message = "长度最小为1,最大为11")
    @NotBlank
    private String phone;	// 电话
    private String sex;
    @Length(min = 1,max = 256,message = "长度最小为1,最大为200")
    @NotBlank
    private String headPic;  //头像
    @Length(min = 1,max = 32,message = "长度最小为1,最大为30")
    @NotBlank
    private String gitLabId; //gitLab账号id
    @Length(min = 1,max = 32,message = "长度最小为1,最大为30")
    @NotBlank
    private String gitLabUsername; //gitLab账号
    @Length(min = 1,max = 32,message = "长度最小为1,最大为30")
    @NotBlank
    private String gitLabImpressionToken; //gitLab账号模拟令牌


    private Date loginDate;	// 最后登陆日期


    public SysUser(String loginName, String delFlag, String flag){
        this.loginName = loginName;
        this.delFlag = delFlag;
        this.flag = flag;
    }

    public SysUser(String loginName, String password, String name, String sex) {
        this.loginName = loginName;
        this.password = password;
        this.name = name;
        this.sex = sex;
    }

    public boolean isEmpty(){
        boolean loginNameIsEmpty = StringUtils.isEmpty(loginName);
        boolean passwordIsEmpty = StringUtils.isEmpty(password);
        boolean nameIsEmpty = StringUtils.isEmpty(name);
        boolean emailIsEmpty = StringUtils.isEmpty(email);
        boolean phoneIsEmpty = StringUtils.isEmpty(phone);
        boolean headPicIsEmpty = StringUtils.isEmpty(headPic);
        if(
                loginNameIsEmpty&&
                        passwordIsEmpty&&
                        nameIsEmpty&&
                        emailIsEmpty&&
                        phoneIsEmpty&&
                        headPicIsEmpty)
            return true;
        else
            return false;
    }


    @Transient
    private List<SysRole> roleList = Lists.newArrayList(); // 拥有角色列表

    /**
     * 权限列表
     * @return
     */
    @JsonIgnore
    public List<SimpleGrantedAuthority> getSimpleAuthorities(){
        List<SimpleGrantedAuthority> authorities = new ArrayList<SimpleGrantedAuthority>();
        SimpleGrantedAuthority auth = null;
        for (SysRole sysRole: roleList) {
            auth = new SimpleGrantedAuthority(sysRole.getCode());
            authorities.add(auth);
        }
        return authorities;
    }
}
