package org.rcisoft.sys.user.service.impl;

import org.rcisoft.core.constant.UserConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class SysUserServiceImpl implements SysUserService {

	@Autowired
	private SysUserMapper sysUserMapper;

	@Autowired
	private PasswordEncoder passwordEncoder;

	@Transactional(propagation = Propagation.REQUIRED,readOnly = false)
	@Override
	public PersistModel addUser(SysUser user) {
		int line = sysUserMapper.insertSelective(user);
		return new PersistModel(line);
	}

	@Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
	@Override
	public PersistModel persistUser(SysUser user) {
		user.setCommonBusinessId();
		user.setPassword(passwordEncoder.encode("123456"));
		int line = sysUserMapper.insertSelective(user);
		return new PersistModel(line);
	}

	@Override
	public void updateUser(SysUser user,String uid,int line) {
		if(!user.isEmpty()){
			user.setBusinessId(uid);
			int l1 = sysUserMapper.updateByPrimaryKeySelective(user);
			if (l1 != line)
				throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
		}
	}

	@Override
	public String checkPassword(String userId) {
		return sysUserMapper.queryPasswordById(userId);
	}

	@Override
	public int changePassword(String userId, String newPassword) {
		return sysUserMapper.changePassword(userId,passwordEncoder.encode(newPassword));
	}

	@Override
	public int resetPassword(String userCode) {
		return sysUserMapper.resetPassword(userCode,passwordEncoder.encode("123456"));
	}
}
